gulp = require 'gulp'

GulpEste = require 'gulp-este'
rimraf = require 'rimraf'
runSequence = require 'run-sequence'
yargs = require 'yargs'

args = yargs
  .alias 'p', 'production'
  .alias 's', 'server'
  .argv

este = new GulpEste __dirname, args.production, '../../../..'

paths =
  coffee: [
    'bower_components/este-library/este/**/*.coffee'
    'src/**/*.coffee'
    'server/src/*.coffee'
  ]
  jsx: [
    'src/**/*.jsx'
  ]
  js: [
    'bower_components/closure-library/**/*.js'
    'bower_components/este-library/este/**/*.js'
    'js/**/*.js'
    'server/js/*.js'
    'tmp/**/*.js'
    '!**/build/**'
  ]
  stylus: [
    'app/styles/*.styl'
  ]

  unittest: [
    'js/**/*_test.js'
  ]
  compiler: 'bower_components/closure-compiler/compiler.jar'
  externs: [
    'app/react-externs.js'
    'app/pouchdb-externs.js'
  ]
  concatAll:
    development: [
      'bower_components/react/react-with-addons.js'
      'bower_components/pouchdb/dist/pouchdb.js'
    ]
    production: [
      'bower_components/react/react-with-addons.min.js'
      'bower_components/pouchdb/dist/pouchdb.min.js'
    ]

dirs =
  googBaseJs: 'bower_components/closure-library/closure/goog'
  watch: ['src', 'js', 'app/styles', 'server']

srcToJs = (path) ->
  path.dirname = path.dirname.replace /src/, (dir) -> 'js'

gulp.task 'clean', (done) ->
  rimraf './js', done

gulp.task 'coffee', ->
  este.coffee paths.coffee, rename: srcToJs

gulp.task 'transpile', (done) ->
  runSequence 'clean', 'coffee', 'stylus', done

gulp.task 'deps', ->
  este.deps paths.js

gulp.task 'unittest', ->
  este.unitTest dirs.googBaseJs, paths.unittest

gulp.task 'dicontainer', ->
  este.diContainer dirs.googBaseJs, [
    name: 'app.DiContainer'
    resolve: ['App']
  ,
    name: 'server.DiContainer'
    resolve: ['server.App']
  ]

gulp.task 'concat-deps', ->
  este.concatDeps()

gulp.task 'compile-clientapp', ->
  este.compile paths.js, 'build',
    compilerPath: paths.compiler
    compilerFlags:
      closure_entry_point: 'app.main'
      compilation_level: 'SIMPLE_OPTIMIZATIONS'
      warning_level: 'VERBOSE'
      jscomp_off: 'globalThis'
      externs: paths.externs

gulp.task 'check', ->
  este.compile paths.js, '/tmp',
    compilerPath: paths.compiler
    compilerFlags:
      closure_entry_point: 'app.main'
      compilation_level: 'ADVANCED_OPTIMIZATIONS'
      warning_level: 'VERBOSE'
      jscomp_off: 'globalThis'
      externs: paths.externs

gulp.task 'compile-serverapp', ->
  este.compile paths.js, '/build',
    compilerPath: paths.compiler
    compilerFlags:
      closure_entry_point: 'server.main'
      externs: paths.externs.concat este.getNodeJsExterns()
    # To have compiled code readable, so we can trace errors.
      debug: true
      formatting: 'PRETTY_PRINT'

gulp.task 'concat-all', ->
  este.concatAll
    'build/app.js': paths.concatAll

gulp.task 'livereload-notify', ->
  este.liveReloadNotify()

gulp.task 'js', (done) ->
  runSequence [
    'deps' if este.shouldCreateDeps()
    'unittest'
    'dicontainer'
    'concat-deps'
    'compile-clientapp' if args.production
    'concat-all'
    'livereload-notify' if este.shouldNotify()
    done
  ].filter((task) -> task)...

gulp.task 'build', (done) ->
  runSequence 'transpile', 'js', done

gulp.task 'livereload-server', ->
  este.liveReloadServer()

gulp.task 'watch', ->
  este.watch dirs.watch,
    coffee: 'coffee'
    js: 'js'
    jsx: 'jsx'
    styl: 'stylus'
    css: 'livereload-notify'
  , (task) -> gulp.start task

gulp.task 'stylus', ->
  este.stylus paths.stylus

gulp.task 'run-server', este.bg 'node', ['server/js/index.js']
gulp.task 'simple-server', este.bg 'node', ['server.js']

gulp.task 'server', ->
  runSequence [
    'run-server'
  ]

gulp.task 'env', ->
  este.setNodeEnv()

gulp.task 'run', (done) ->
  runSequence [
    'env'
    'livereload-server' if !args.production
    'watch'
    'server' if args.server
    'simple-server' unless args.server
    done
  ].filter((task) -> task)...

gulp.task 'default', (done) ->
  runSequence 'build', 'run', ->
    done()
    console.log 'Point your browser to http://localhost:8000'

gulp.task 'bump', (done) ->
  este.bump './*.json', yargs, done
