goog.provide 'app.formatter.Document'

goog.require 'app.BlockType'
#goog.require 'goog.text.LoremIpsum'
goog.require 'app.formatter.ImagesFormatter'

class app.formatter.Document

  @MinFontSize: 10
  @MaxFontSize: 14
  @PageWidth: '20cm'

  ###*
    @param {app.formatter.Store} store
    @param {app.formatter.FontCalc} fontCalc
    @param {app.DocumentStore} docStore
    @constructor
  ###
  constructor: (store, fontCalc, docStore) ->
    {
      h1, h2, h3, h4, h5, h6
      div, span, section
      p
      input, textarea
      button
    } = React.DOM
    # to model

#    loremIpsum = new goog.text.LoremIpsum()
#    loremIpsum.generateParagraph()
    loremParagraph = 'Rhoncus. Orci malesuada lectus, morbi rhoncus litora ut nascetur. Pellentesque conubia curae per vestibulum. Mus scelerisque feugiat sollicitudin blandit. Cum urna urna mus felis proin, mollis sapien risus ipsum primis. Mi lectus sollicitudin elementum litora aenean, vulputate id, malesuada arcu. Leo, proin phasellus lacinia pellentesque fermentum nulla in lacus. Pede consequat fringilla urna suspendisse natoque, leo. Nam elit dui vitae velit. Ridiculus duis, nisl penatibus sociosqu, libero nisl cubilia ipsum ve arcu, odio arcu dis netus. At. Tortor. Ac, facilisis purus ve nisl.'

    ImagesFormatterView = app.formatter.ImagesFormatter.reactClass

    @reactClass = React.createClass(
      displayName: 'document'

      componentDidMount: ->
        store.listen app.formatter.Store.EventType.Change, @onStoreChange

      onStoreChange: ->
        @forceUpdate() if @isMounted()

      render: ->
        @formatSettings = store.getSettings()

        docBlock = docStore.getDocument()
        images = docStore.findOfType(app.BlockType.Image)
        div(
          {style: {width: app.formatter.Document.PageWidth}}
#          @renderDocument(docBlock)
          @fontSizesChooser()
          @fontSizeView()
          ImagesFormatterView({images: images, onImageFormatChosen: @imageFormatChosen})
          @fontFamilyChooser()
        )

      ###*
        @param {app.document.Image} image
        @param {app.formatter.ImageSettings.Format} format
      ###
      imageFormatChosen: (image, format) ->
        store.changeImageFormat(image, format)


      fontFamilyChooser: ->
        return div(
          {className: 'ff-chooser'}
          h2({}, 'Font family')
          Object.keys(app.formatter.DocumentSettings.FontFamily).map @fontFamilyView
        )

      onFontFamilyChoose: (fontKey) ->
        store.changeFontFamily fontKey

      fontFamilyView: (fontKey) ->
        active = if (@formatSettings.fontFamily == fontKey) then 'active-ff' else ''
        div(
          {
            onClick: => @onFontFamilyChoose(fontKey)
            className: "ff-view #{active}"
            style: {fontFamily: @getActualFFValue(fontKey)}
          }
          h2({}, app.formatter.DocumentSettings.FontFamily[fontKey])
          loremParagraph
        )

      fontSizeView: ->
        firstTextBlock = docStore.findFirstOfType(app.BlockType.Paragraph)
        text = if firstTextBlock?.text then firstTextBlock.text else loremParagraph
        styles = {
          fontSize: "#{@formatSettings.fontSize}pt"
          fontFamily: @formatSettings.fontFamily
        }
        div(
          {}
          h6({}, 'text example: ')
          p(
            {
              style: styles
              className: 'example'
            }
            text
          )
        )

      fontSizesChooser: ->
        {MinFontSize, MaxFontSize} = app.formatter.Document
        div(
          {}
          h2({}, 'Text font size')
          span({}, 'Choose text font size: ')
          div(
            {className: 'font-chooser'}
            MinFontSize
            input(
              {
                type: 'range'
                min: MinFontSize
                max: MaxFontSize
                step: 1
                value: @formatSettings.fontSize
                onChange: (e) -> store.changeTextSize(e.target.value)
              }
              MaxFontSize
            )
            span(
              {className: 'actual-size'}
              "Actual size is #{@formatSettings.fontSize} pt."
            )
          )
        )

      getActualFFValue: (fontKey) ->
        app.formatter.DocumentSettings.FontFamilyValue[fontKey]

      renderDocument: (doc) ->
        h1(
          {
            className: 'document'
          }
          doc.title
        )

    )
