goog.require 'tests.Factory'
goog.require 'app.document.Reference'

assertIsPromise = (object) ->
  assert.ok object.then

assertIsReference = (object) ->
  assert.ok object._id
  assert.equal object.title, ''
  assert.ok object.type

promise = {then: -> promise}

suite 'app.references.Store', ->

  setup ->
    db = {get: -> promise}
    @store = new app.references.Store(db)

  test 'init ok', ->
    assert.ok @store

  suite 'makeReference', ->

    test 'creates link', ->
      link = @store.makeReference({type: app.document.Reference.Type.Link})
      assertIsReference(link)
      assert.equal link.type, app.document.Reference.Type.Link
      assert.equal link.url, ''


    test 'creates term', ->
      ref = @store.makeReference({type: app.document.Reference.Type.Term})
      assertIsReference(ref)
      assert.equal ref.type, app.document.Reference.Type.Term
      assert.equal ref.description, ''


    test 'creates abbreviation', ->
      ref = @store.makeReference({type: app.document.Reference.Type.Abbreviation})
      assertIsReference(ref)
      assert.equal ref.type, app.document.Reference.Type.Abbreviation
      assert.equal ref.fullTitle, ''


    test 'creates citation', ->
      ref = @store.makeReference({citation: 'citation/text', type: app.document.Reference.Type.Citation})
      assertIsReference(ref)
      assert.equal ref.type, app.document.Reference.Type.Citation
      assert.equal ref.citation, 'citation/text'


  suite 'references access', ->

    suite 'getters', ->
      setup ->
        @store.references = [
          {type: app.document.Reference.Type.Term}, {type: app.document.Reference.Type.Link}
          {type: app.document.Reference.Type.Link}, {type: app.document.Reference.Type.Citation}
          {type: app.document.Reference.Type.Abbreviation}, {type: app.document.Reference.Type.Abbreviation}
        ]
#      test 'getTerms', -> assert.lengthOf @store.getTerms(), 1
      test 'getReferencesOrPromise', ->
        assertIsPromise @store.getReferencesOrPromise()

  suite 'persistence', ->
    setup ->
      @store.db = {
        put: ->
      }
      @mock = sinon.mock(@store.db)

    teardown ->
      @mock.verify()

    test '_save', ->
      @mock.expects('put').once().returns(promise)
      @store._save()


  suite 'functions that saves and notifies', ->
    setup ->
      @store._notify = sinon.spy()
      @store._save = sinon.spy()

    teardown ->
      assert.ok @store._notify.calledOnce
      assert.ok @store._save.calledOnce

    test 'changeReference', ->
      ref = {
        propertyToChange: 'old value'
      }
      @store.changeReference(ref, {propertyToChange: 'new value'})

    test 'changeTitle', ->
      ref = {title: 'old title'}
      @store.changeTitle(ref, 'new one')
      assert.equal ref.title, 'new one'
