goog.provide 'app.viewer.Block'

goog.require 'apptypes'
goog.require 'app.viewer.List'
goog.require 'app.viewer.Image'
goog.require 'app.BlockType'

class app.viewer.Block

  ###*
    @param {app.formatter.FontCalc} fontCalc
    @param {app.Routes} routes
    @constructor
  ###
  constructor: (fontCalc, routes) ->
    {
      h1, h2, h3, h4, h5, h6
      aside
      div, p, a
    } = React.DOM
    ListViewer = app.viewer.List.reactClass
    ImageViewer = app.viewer.Image.reactClass

    levelToHeader = {
      1: h1
      2: h2
      3: h3
      4: h4
      5: h5
      6: h6
    }

    ###*
      @type {function(apptypes.BlockViewerProps)}
    ###
    @reactClass = React.createClass(
      displayName: 'DocumentViewer'

      getDefaultProps: ->
        return {
          levelCount: 0
          level: 1
          activeBlockId: false
        }

      componentDidUpdate: ->
        @scrollToElIfActive()

      componentDidMount: ->
        @scrollToElIfActive()

      ###*
        Scrolls to the block if is active (e.g. clicked from outline or redirected from
        other module.
      ###
      scrollToElIfActive: ->
        blockId = @props.activeBlockId

        if @props.block._id == blockId
          console.log  ' is active', @props.block if app.debug
          el = @getDOMNode()
          el.scrollIntoView(true)

      processParagraph: (block) ->
        paragraphParts = block.text.split(/(\n)+/)
        paragraphs = paragraphParts.filter (part) -> part != '\n'
        return paragraphs.map (paragraph) => p({}, paragraph)

      render: ->
        ##
        formatSettings = @props.formatSettings

        fontCalc.setAllValues({
          baseSize: formatSettings.fontSize
        })

        block = @props.block
        level = @props.level
        if level > 6 then console.error 'Program error occured, max header level should be 6.'

        actualHeader = levelToHeader[level]
        throw new Error 'no header function' unless typeof actualHeader == 'function'
        # block doesnt have to have a title (with value)
        if block.title
          if block.type == app.BlockType.Section
            headerStyle = {fontSize: fontCalc.headerSize(level) + "pt" }
            optionalHeader = actualHeader(
              {style: headerStyle}
              "#{block.title}"
            )
          else if block.type == app.BlockType.Document
            optionalHeader = h1({className: 'document-title'}, block.title)
          else if block.type != app.BlockType.Image
            optionalHeader = h6(
              {style: {fontSize: "#{formatSettings.fontSize}pt", fontWeight: 'bold'}}
              block.title
            )



        optionalComment = aside({}, block.comment) if block.comment

        text = switch block.type
          when app.BlockType.Paragraph
           @processParagraph(block)
          when app.BlockType.List
            ListViewer({items: block.items, isOrdered: block.isOrdered})
          when app.BlockType.Image
            ImageViewer({image: block})
          when app.BlockType.Section
            console.warn('no section rendering') if app.lamebug
          else
            console.error('block type expected') if block.type != app.BlockType.Document

        return div(
          {
            className: ''
          }
          optionalHeader
          optionalComment
          text
          @renderNestedFirst()
        )


      renderNestedFirst: ->
        block = @props.block
        levelCount = @props.levelCount
        nestedBlocks = null

        if block.blocks?
          # smell code
          levelCount = 0
          nestedBlocks = block.blocks.map (newBlock) =>
            @props.renderBlocks(levelCount + 1, @props.level + 1, newBlock)

        return nestedBlocks
    )
