goog.provide 'app.viewer.Document'

goog.require 'app.viewer.Outline'
goog.require 'app.document.Block'
goog.require 'app.formatter.Store'
goog.require 'app.formatter.Document'
goog.require 'app.document.Document'

class app.viewer.Document

  ###*
    @param {app.DocumentStore} documentStore
    @param {app.formatter.Store} formatStore
    @param {app.viewer.Block} blockViewer
    @param {app.Routes} routes
    @constructor
  ###
  constructor: (@documentStore, @formatStore, blockViewer, routes) ->
    {
      html,head,meta,title,link,body,style
      article,aside
      h1
      div
      textarea, button, a
    } = React.DOM
    BlockViewer = blockViewer.reactClass
    Outline = app.viewer.Outline.reactClass
    documentStore = @documentStore
    formatStore = @formatStore

    @reactClass = React.createClass(
      displayName: 'DocumentViewer'

      getInitialState: ->
        return {
          doc: documentStore.getDocument()
        }

      render: ->
        div(
          {
            id: 'document-viewer'
          }
          div(
            {className: 'outline'}
            Outline({outline: @state.doc, routes: routes})
          )
          div(
            {
              className: 'view'
              style: {fontSize:  @state.doc.formatSettings.fontSize + "pt"}
            }
            @renderInfo()
            @renderSaveBtn()
#            a({href: '/document.html', target: '_blank'}, 'save from server')
            @state.doc.blocks.map @renderRecursively.bind(null, 0, 1)
          )
        )

      renderDocument: ->
        doc = documentStore.getDocument()
        fontSizeValue = doc.formatSettings.fontSize

        bodyStyles = {
          backgroundColor: 'rgb(243, 243, 243)'
        }

        articleStyles = {
          fontFamily: doc.formatSettings.fontFamily
          fontSize: "#{fontSizeValue}pt"
          maxWidth: app.formatter.Document.PageWidth
        }

        globalStyleString = """
          aside {
            float: right;
            width: 20%;
            background-color: #000;
          }
          article {
            background-color: #fff;
            color: #000;
            max-width: 20cm;
            margin: 3em auto;
            padding: 2em;
            text-align: left;
          }
          p,
          ul,
          ol,
          img,
          h1,
          h2,
          h3,
          h4,
          h5,
          h6 {
            width: 60%;
          }
          p,
          ul,
          ol {
            line-height: 1.2em;
          }
          h1,
          h2,
          h3,
          h4,
          h5 {
            font-weight: normal;
            margin: 1.2em 0 0.3em 0;
          }
          p {
            margin: 1em 0;
          }
          h6 {
            font-weight: bold;
            margin: 2em 0 0 0;
          }
          h6 + p,
          h6 + ul,
          h6 + ol {
            margin-top: 0.5em;
          }
          h1.document-title {
            color: #929292;
          }
        """

        return html(
          {}
          head(
            {}
            meta(
              {charSet: 'utf-8'}
              title({}, "Document #{doc.title}")
            )
            style(
              {dangerouslySetInnerHTML: {__html: globalStyleString}}

            )
          )
          body(
            {style: bodyStyles}
            article(
              {style: articleStyles}
              @renderRecursively(0, 1, @state.doc)
            )
          )
        )

      ###*
        Renders only info about document.
      ###
      renderInfo: ->
        h1(
          {
            className: 'document'
          }
          @state.doc.title
        )

      ###*
        @param {number} levelCount - total levels
        @param {number} level - total levels
        @param {app.document.Block} block
      ###
      renderRecursively: (levelCount, level, block) ->
        return BlockViewer({
          block
          formatSettings: formatStore.getSettings()
          levelCount
          level
          activeBlockId: @props.viewedBlockId
          renderBlocks: @renderRecursively
          isActive: @isBlockActive(block)
        })

      renderSaveBtn: ->
        button({href: '', onClick: @prepareToSave}, 'save document as file')

      ###*
        @param {app.document.Block} block
      ###
      isBlockActive: (block) ->
        return (block._id == @props.viewedBlockId)

      ###*
        TODO: mixin
      ###
      onStoreChange: ->
        if @isMounted()
          @forceUpdate()
        else
          console.warn 'Not mounted on store change' if app.debug

      fireClick: (el) ->
          evObj = document.createEvent("Events")
          evObj.initEvent 'click', true, false
          el.dispatchEvent evObj

      ###*
        @param {Object} e
      ###
      prepareToSave: (e) ->
        console.log 'prepareToSave' if app.debug
        aClicked = e.target
        HTMLReactElement = @renderDocument()
#        BlockViewer({
#          block: @state.doc
#
#          renderBlocks: @renderRecursively
#          levelCount: 0
#          level: 1
#        })
        htmlString = React.renderComponentToStaticMarkup(HTMLReactElement)
        htmlString = '<!DOCTYPE html>' + htmlString
        downloadLink = document.createElement('a')
        downloadLink.innerHTML = 'download file'
        downloadLink.href = "data:text/html;charset=UTF-8,#{htmlString}"
        downloadLink.download = 'generated_document.html'

        aParent = aClicked.parentElement
        aParent.replaceChild(downloadLink, aClicked)
        downloadLink.addEventListener('click', -> aParent.replaceChild(aClicked, downloadLink))

        try
          @fireClick downloadLink
        catch error
          console.log error



    )
